/**
 * Register a new event to move the converse element
 * into another component.
 */
converse.plugins.add('sib-chat-navigation', {
  initialize() {
    const _converse = this._converse;
    const { api } = _converse;

    window.addEventListener('sib-change-chat', async ev => {
      let { jid, is_groupchat, root } = ev.detail;

      if (!jid) {
        return;
      }

      jid = jid.toLowerCase();
      _converse.root = root;

      // Find the converse root
      const converse_el = Array.from(document.querySelectorAll('solid-xmpp-chat'))
        .map(el => el.shadowRoot.getElementById('conversejs'))
        .filter(el => el)
        .pop();

      if (converse_el) {
        root.appendChild(converse_el);
      }

      if (is_groupchat) {
        await api.rooms.open(jid, {}, true);
      } else {
        await api.chats.open(jid, {}, true);
      }
    });

    // Get the currently used solid-xmpp-chat
    const getCurrentChat = () => Array.from(document.querySelectorAll('solid-xmpp-chat'))
      .filter(el => el.shadowRoot.getElementById('conversejs'))
      .pop();

    // When the a chat box is focused, send the read event
    api.listen.on('chatBoxFocused', function() {
      const resource = getCurrentChat()?.component.resource;

      window.dispatchEvent(new CustomEvent('read', {
        detail: {
          resource,
        },
      }));
    });
  },
});
