import { Picker } from 'emoji-picker-element';
/**
 * Reactions plugin for Converse.
 * TODO: Watch out for performances
 * https://xmpp.org/extensions/xep-0444.html
 */
converse.plugins.add('sib-reactions', {
  overrides: {
    ChatRoom: {
      async onMessage(attrs) {
        await this.onMessageWithReaction.apply(this, arguments);
        return await this.__super__.onMessage.apply(this, arguments);
      },
    },
    ChatBox: {
      async onMessage(attrs) {
        await this.onMessageWithReaction.apply(this, arguments);
        return await this.__super__.onMessage.apply(this, arguments);
      },
      async onMessageWithReaction(attrs) {
        const { _ } = converse.env;
        attrs = await attrs;

        // Update message model with the reactions
        if (attrs.reaction_id) {
          const message = this.messages.findWhere({
            'id': attrs.reaction_id,
          });

          if (message) {
            const reactions = message.get('reactions') || {};
            const from_jid = attrs.from_muc ? attrs.from_real_jid : attrs.from;

            // If there are reactions for this user, replace with the latest ones
            reactions[from_jid] = {
              emojis: attrs.reaction_emojis,
              origin_id: attrs.id,
              from_jid,
            };

            let reactions_order = message.get('reactions_order') || [];
            reactions_order = _.uniq(reactions_order.concat(attrs.reaction_emojis));

            // Remove the emojis that are not in here anymore
            const all_emojis = Object.values(reactions).map(r => r.emojis).flat();
            reactions_order = reactions_order.filter(e => all_emojis.includes(e));

            // Forces refresh
            message.set({ 'reactions': {}, 'reactions_order': [] });
            message.save({ reactions, reactions_order });
          }
        }
      },
    },
  },
  initialize() {
    const _converse = this._converse;
    const { api, __ } = _converse;
    const { $msg, u, Strophe, sizzle, html, _, log } = converse.env;

    Strophe.addNamespace('REACTIONS', 'urn:xmpp:reactions:0');

    /**
     * Document callback (when clicking outside of the picker)
     * TODO: This is a quick and dirty implementation, needs update
     */
    let documentCallback;

    /**
     * Send a reaction stanza for this user with the given emojis.
     * @param message
     * @param emojis
     * @param update
     */
    function sendReactionStanza(message, emojis, update = true) {
      const msgid = message.get('msgid');
      const type = message.get('type');
      const to = message.get(type === 'groupchat' ? 'from_muc' : 'contact_jid');

      const msg = $msg({
        'id': u.getUniqueId(),
        'to': to,
        'type': type,
      })
        .c('store', {
          'xmlns': Strophe.NS.HINTS,
        })
        .up()
        .c('reactions', {
          'id': msgid,
          'xmlns': Strophe.NS.REACTIONS,
        });

      // Insert reaction emojis into the stanza
      emojis.forEach(em => {
        msg.c('reaction').t(em).up();
      });

      // Send reaction stanza
      // TODO: Await?
      _converse.connection.send(msg);

      // Force UI update in the model
      if (update) {
        const from_jid = _converse.bare_jid;
        const reactions = message.get('reactions') || {};
        reactions[from_jid] = {
          emojis,
          origin_id: reactions[from_jid]?.origin_id || null,
          from_jid,
        };

        let reactions_order = message.get('reactions_order') || [];
        reactions_order = _.uniq(reactions_order.concat(emojis));

        // Remove the emojis that are not in here anymore
        const all_emojis = Object.values(reactions).map(r => r.emojis).flat();
        reactions_order = reactions_order.filter(e => all_emojis.includes(e));

        message.set({ 'reactions': {}, 'reactions_order': [] });
        message.save({ reactions, reactions_order });
      }
    }

    /**
     * Get the current user emojis for a given message.
     * @param message
     */
    function getUserReactionEmojis(message) {
      const reactions = message.get('reactions') || {};
      const from_jid = _converse.bare_jid;
      const reaction = reactions[from_jid];
      return reaction?.emojis || [];
    }

    /**
     * Extends the Message element.
     * @type {*}
     */
    const MessageMixin = {
      renderMessageText() {
        return html`
          ${_renderMessageText.call(this)}
          ${this.hasReactions() ? this.renderReactions() : ''}
        `;
      },

      /**
       * Render the reactions list.
       * Adds an additional icon to add a reaction.
       * @returns {html}
       */
      renderReactions() {
        return html`
          <div class="chat-msg__reactions">
            ${this.getEmojisByCount().map(({ emoji, count, from_user }) => html`
              <a href="#" class="chat-msg__reaction ${from_user ? 'chat-msg__reaction--me' : ''}"
                 @click="${ev => this.toggleReaction(ev, emoji)}">
                <span class="emoji">${emoji}</span>
                <span class="count">${count}</span>
              </a>
            `)}
            <a href="#" @click="${this.openEmojiPicker}" class="chat-msg--react"><i class="fa fa-smile"></i></a>
          </div>
        `;
      },

      /**
       * Returns TRUE if the message has reactions.
       * Sometimes the reactions array contains empty reactions list
       * when a someone removes all of their emojis.
       * @returns {boolean}
       */
      hasReactions() {
        const reactions = this.model.get('reactions');
        return reactions ? Object.values(reactions).map(r => r.emojis).flat().length > 0 : false;
      },

      /**
       * Returns an object with the emoji as key and its count for value.
       * @returns {*}
       */
      getEmojisByCount() {
        const reactions = this.model.get('reactions');
        const from_jid = _converse.bare_jid;
        const emojis = {};

        const order = this.model.get('reactions_order') || [];
        for (let e of _.values(order)) {
          emojis[e] = {
            emoji: e,
            count: 0,
            from_user: false,
          };
        }

        for (let r of _.values(reactions)) {
          for (let e of r.emojis) {
            if (emojis.hasOwnProperty(e)) {
              emojis[e].count += 1;
              emojis[e].from_user = emojis[e].from_user || r.from_jid === from_jid;
            } else {
              emojis[e] = {
                emoji: e,
                count: 1,
                from_user: r.from_jid === from_jid,
              };
            }
          }
        }

        return _.values(emojis);
      },

      /**
       * Open the emoji drawer with all the needed stuff.
       * - When the user selects an emoji, it sends the reaction stanza.
       * - When the user press escape, it closes the drawer.
       * - When the user click outside of the drawer, it closes it.
       * @param ev
       */
      openEmojiPicker(ev) {
        ev.preventDefault();

        // Avoid an error when opening a new picker
        if (document.querySelector('.emoji-reaction-select')) {
          return;
        }

        const picker = new Picker();
        picker.classList.add('light', 'emoji-reaction-select');

        function closePicker() {
          picker.remove();
          document.removeEventListener('click', documentCallback);
        }

        picker.addEventListener('emoji-click', e => {
          const emoji = e.detail.unicode;
          const emojis = getUserReactionEmojis(this.model);

          // TODO: If it has the reaction from this user already, remove it? (config)
          if (emojis?.includes(emoji)) {
            log.warn(`The emoji "${emoji}" is already in the list of this user reactions`);
            closePicker();
            return;
          }

          emojis.push(emoji);
          closePicker();

          sendReactionStanza(this.model, emojis);
        });

        // Close the picker if the ESC key is pressed
        picker.addEventListener('keydown', ev => {
          if (ev.key === 'Escape') {
            closePicker();
          }
        });

        // Close the picker if clicked outside of it
        setTimeout(() => {
          documentCallback = ev => {
            if (!picker.shadowRoot.contains(ev.composedPath()[0])) {
              closePicker();
            }
          };
          document.addEventListener('click', documentCallback);
        });

        // Append picker to the document
        // Note: It must be the body since the shadowRoot is misplaced when switching room (?)
        document.body.appendChild(picker);

        // Position the emoji picker according to the current target
        // TODO: Clean this part, maybe look for a lib that does the job
        // TODO: Add transition animation when showing
        let tr = ev.target.getBoundingClientRect();
        let docEl = document.documentElement;
        let rectTop = tr.top + window.pageYOffset - docEl.clientTop;
        let rectLeft = tr.left + window.pageXOffset - docEl.clientLeft;

        picker.style.position = 'fixed';
        picker.style.zIndex = '99999';

        let pr = picker.getBoundingClientRect();

        // Check if the viewport is mobile friendly
        // We have to do this here since the styles is scoped to the SXC component shadow DOM
        const mql = window.matchMedia('(max-width: 767.98px)');

        if (mql.matches) {

          // MOBILE
          picker.style.left = '0';
          picker.style.bottom = '0';
          picker.style.width = '100%';
          picker.style.height = '60vh';
          picker.style.boxShadow = '0 0 0 100vh rgba(0, 0, 0, 0.3)';
          picker.style.setProperty('--emoji-size', '1.6rem');
        } else {

          // DESKTOP
          // Stick on the left side of the target
          // Stick on the right side if there is not enough space for it
          // Center if there are no space anywhere
          picker.style.margin = '10px';
          picker.style.boxShadow = '0 0 10px rgba(0, 0, 0, 0.2)';
          picker.style.borderRadius = '3px';

          if (window.innerWidth - (rectLeft + tr.width) > pr.width) {
            picker.style.left = `${rectLeft + tr.width}px`;
          } else if (rectLeft > pr.width) {
            picker.style.right = `${window.innerWidth - rectLeft}px`;
          } else {
            picker.style.left = '50%';
            picker.style.transform = 'translateX(-50%)';
            picker.style.margin = '0px';
          }

          // Add an offset so it's a bit higher than the target
          const offY = 100;

          // Show the drawer a bit higher than the target
          // Stick at the bottom if there are no space available
          if (pr.height + (rectTop - offY) > window.innerHeight) {
            picker.style.bottom = `0px`;
          } else {
            picker.style.top = `${rectTop - offY}px`;
          }
        }

        // Focus the search input when the picker opens
        const emojiSearch = picker.shadowRoot?.querySelector('.search');
        emojiSearch?.focus();
      },

      /**
       * Add an emoji reaction on the message if another user added it.
       * Remove an emoji reaction from the message if the user added it.
       * @param ev
       * @param emoji
       */
      toggleReaction(ev, emoji) {
        ev.preventDefault();
        let emojis = getUserReactionEmojis(this.model);

        if (emojis?.includes(emoji)) {
          emojis = emojis.filter(em => em !== emoji);
        } else {
          emojis.push(emoji);
        }

        sendReactionStanza(this.model, emojis);
      },
    };

    // Apply message mixin
    const Message = _converse.api.elements.registry['converse-chat-message'];
    const _renderMessageText = Message.prototype.renderMessageText;
    Object.assign(Message.prototype, MessageMixin);

    /**
     * Register new reaction attributes.
     *
     * @param stanza
     * @param attrs
     * @returns {*}
     */
    function parseMessage(stanza, attrs) {
      const reactions = sizzle(`reactions[xmlns="${Strophe.NS.REACTIONS}"]`, stanza).pop();

      if (reactions) {
        attrs.reaction_id = reactions.getAttribute('id');
        attrs.reaction_emojis = Array.from(reactions.querySelectorAll('reaction')).map(el => el.textContent);
      }

      // Resolve the correct jid if not found in occupants
      if (reactions && attrs.from_real_jid === undefined) {
        const x = sizzle(`x[xmlns="${Strophe.NS.MUC_USER}"]`, stanza).pop();
        if (x) {
          attrs.from_real_jid = x.querySelector('item')?.getAttribute('jid');
        }
      }

      return attrs;
    }

    // Register handler for MUCs and chats
    api.listen.on('parseMessage', parseMessage);
    api.listen.on('parseMUCMessage', parseMessage);

    /**
     * Open the emoji drawer from message actions.
     * This is the first method of adding an emoji if there are none yet.
     */
    api.listen.on('getMessageActionButtons', (el, buttons) => {
      if (el.is_retracted || el.correcting) {
        return buttons;
      }

      buttons.push({
        'i18n_text': __('React'),
        'handler': ev => MessageMixin.openEmojiPicker.call(el, ev),
        'button_class': 'chat-msg__action-react',
        'icon_class': 'fa fa-smile',
        'name': 'react',
      });

      return buttons;
    });
  },
});
