/**
 * Add an action button to reply to a message.
 * TODO: Add settings to handle this plugin
 */
converse.plugins.add('sib-reply-to-message', {
  initialize() {
    const _converse = this._converse;
    const { api, __ } = _converse;
    const { utils, _ } = converse.env;

    function removeQuotedMessages(text) {
      return text.replace(/^>.*(\n)?/gm, '');
    }

    api.listen.on('getMessageActionButtons', (el, buttons) => {

      // Do not add the reply button if the message is retracted or correcting
      if (el.is_retracted || el.correcting) {
        return buttons;
      }

      buttons.push({
        'i18n_text': __('Reply'),
        'handler': ev => {
          const chat_textarea = utils.ancestor(el, '.chatbox')?.querySelector('.chat-textarea');
          chat_textarea.value = '';

          // Add the previous message without quoted messages (to avoid blockquotes into blockquotes)
          const message = _.truncate(removeQuotedMessages(el.model.get('message')), { length: 100 });
          if (message.length > 0) {
            chat_textarea.value += `${message.replace(/^/gm, '>')}\n`;
          }

          // Add mention to the initial author for MUCs
          // Do not add the mention if you were the author of the original message
          const nickname = el.model.get('nick');
          if (el.model.get('type') === 'groupchat' && el.model.get('sender') !== 'me' && nickname.length) {
            chat_textarea.value += `@${nickname} `;
          }

          chat_textarea.focus();
        },
        'button_class': 'chat-msg__action-reply',
        'icon_class': 'fa fa-reply',
        'name': 'reply',
      });

      return buttons;
    });
  },
});
