/**
 * Allow the user to retract a file upload.
 */
converse.plugins.add('sib-retract-file-upload', {
  initialize() {
    const _converse = this._converse;
    const { api, Message } = _converse;

    api.settings.extend({
      'allow_upload_retraction': 'own',
    });

    const MessageMixin = {
      /**
       * The message can be retracted if the above option is correctly set.
       * Overrides the 'allow_message_retraction' setting if disabled.
       *
       * @private
       * @method _converse.Messages#mayBeRetracted
       * @returns { Boolean }
       */
      mayBeRetracted() {
        return _mayBeRetracted.call(this) || (
          this.get('sender') === 'me'
          && this.get('oob_url')
          && ['all', 'own'].includes(api.settings.get('allow_upload_retraction'))
        );
      },
    };

    const _mayBeRetracted = Message.prototype.mayBeRetracted;
    Object.assign(Message.prototype, MessageMixin);
  },
});
