/**
 * Forces the active chat to show the new messages indicator
 * when the tab is focused again and we where previously scrolled down.
 * TODO: Rename this plugin
 */
converse.plugins.add('sib-scroll-down-on-focus', {
  overrides: {
    ChatBoxView: {
      /**
       * Force the chat view to be set as active when visible.
       * Like it's set on "inactive" when the tab is focused out.
       * @param {string} state
       */
      onWindowStateChanged(state) {
        const { _converse } = this.__super__;

        if (state === 'visible' && !this.model.isHidden()) {
          this.model.setChatState(_converse.ACTIVE, {
            'silent': true,
          });
        }

        this.__super__.onWindowStateChanged.apply(this, arguments);
      },

      /**
       * @inheritDoc
       */
      onMessageAdded(message) {
        const { _converse } = this.__super__;

        // On new message, set the chat view as scrolled
        // so the new messages indicator is properly displayed
        // when the view is inactive (tab not focused).
        if (this.model.get('chat_state') === _converse.INACTIVE) {
          this.model.set('scrolled', true);
        }

        this.__super__.onMessageAdded.apply(this, arguments);
      },
    },
  },
  initialize() {
    const _converse = this._converse;
    const { api } = _converse;

    // Return the current root component
    function getRootComponent() {
      return api.settings.get('root')?.getRootNode()?.host?.component;
    }

    // When the chat view has been scrolled to the bottom
    // send the read event to remove SIB notifications in UI
    api.listen.on('chatBoxScrolledDown', (data) => {
      if (data.chatbox.isHidden()) {
        return;
      }

      window.dispatchEvent(new CustomEvent('read', {
        detail: {
          resource: {
            '@id': getRootComponent()?.resource['@id'],
          },
        },
      }));
    });
  },
});
