import('../utils.js').then((utils) => {
  import(/* @vite-ignore */ utils.coreVersion()).then(async (core) => {
    /**
     * Initialize rai plugin.
     */
    converse.plugins.add('sib-subscribe-to-rai', {
      dependencies: ['converse-rai'],
      async initialize() {
        const { log } = converse.env;
        const _converse = this._converse;
        const { api } = _converse;

        /**
         * Return the user's memberships as a set of jabberIDs.
         *
         * @param {'circles' | 'projects'} entity
         * @returns {Promise<string[]>}
         */
        const fetchMemberships = async (entity) => {
          const user = await document.querySelector('sib-auth').getUser();

          // FIXME: The context is undefined
          const userProfile = await core.store.getData(
            user['@id'],
            this.context
          );
          const entities = await userProfile[entity];

          const res = await core.store.fetchAuthn(entities['@id'], {
            headers: {
              depth: 1,
              'accept-model-fields': '["jabberID"]',
            },
          });

          const data = await res.json();
          return (
            data['ldp:contains']?.flatMap((item) => item['jabberID']) || []
          );
        };

        const onConnected = async () => {
          const userRooms = (
            await Promise.all([
              fetchMemberships('circles'),
              fetchMemberships('projects'),
            ])
          )
            .flat()
            .filter(Boolean);

          log.info(`User rooms: ${userRooms.join(', ')}`);
          await api.rooms.subscribe(userRooms);
        };

        // Send RAI subscriptions when connected to the XMPP endpoint
        api.listen.on('connected', onConnected);
        api.listen.on('reconnected', onConnected);

        // Set timeout so the event is sent once the document is loaded
        // Await document ready state so the event is properly registered
        api.listen.on('chatRoomHasActivity', async (jid) => {
          await utils.documentReady();
          setTimeout(() => {
            window.dispatchEvent(
              new CustomEvent('newMessage', {
                detail: {
                  jid,
                },
              })
            );
          }, 250);
        });
      },
    });
  });
});
