import style from "./styles/index.scss?inline";
import("./utils.js").then((utils) => {
  import(/* @vite-ignore */ utils.coreVersion()).then(async (core) => {
    await import("./conversejs/converse.js");
    await import("./conversejs/emojis.js");

    await import("./plugins/converse-rai.js");
    await import("./plugins/sib-action-buttons-inline.js");
    await import("./plugins/sib-chat-navigation.js");
    await import("./plugins/sib-custom-hats.js");
    await import("./plugins/sib-disconnected.js");
    await import("./plugins/sib-emoji-picker.js");
    await import("./plugins/sib-history-improved.js");
    await import("./plugins/sib-mention-autocomplete.js");
    await import("./plugins/sib-mention-mobile.js");
    await import("./plugins/sib-reactions.js");
    await import("./plugins/sib-remove-notifications.js");
    await import("./plugins/sib-reply-to-message.js");
    await import("./plugins/sib-retract-file-upload.js");
    await import("./plugins/sib-scroll-down-on-focus.js");
    await import("./plugins/sib-send-button-mobile.js");
    await import("./plugins/sib-subscribe-to-rai.js");

    core.Sib.register({
      name: "solid-xmpp-chat",
      use: [core.StoreMixin],
      attributes: {
        authentication: {
          type: String,
          default: "login",
        },
        autoLogin: {
          type: Boolean,
          default: true,
        },
        i18n: {
          type: String,
          default: "fr",
        },
        websocketUrl: {
          type: String,
          default: "wss://jabber.happy-dev.fr/xmpp-websocket",
        },
        debug: {
          type: Boolean,
          default: false,
        },
      },

      get extra_context() {
        return {
          foaf: "http://xmlns.com/foaf/0.1/",
          chatProfile: "https://cdn.startinblox.com/owl#chatProfile",
          jabberID: "foaf:jabberID",
        };
      },

      async created() {
        const check_identified = await document
          .querySelector("sib-auth")
          .getUserIdToken();
        if (check_identified) {
          this.element.attachShadow({ mode: "open" });
          if (typeof converse_sib === "undefined") {
            this.initializeConverse();
          }
        } // Else, not logged in on page load (even if not on chat)
      },

      importCSS(css) {
        if (Array.from(this.element.shadowRoot.querySelectorAll('style')).length == 0) {
          let link = document.createElement("style");
          link.innerHTML = css;
          this.element.shadowRoot.append(link);
        }
      },

      empty() {
        this.element.innerHTML = "Loading...";
      },

      async populate() {
        if (typeof converse_sib !== "undefined") {
          await converse_sib.loaded_deferred;
          if (this.resource) {
            if (await this.resource.jabberRoom) {
              this.jid = await this.resource["jabberID"];
            } else {
              this.jid = await this.resource["chatProfile.jabberID"];
              const user = await document.querySelector("sib-auth").getUser();
              const userProfile = await core.store.getData(
                user["@id"],
                this.context
              );
              const contactsURL = await userProfile["contacts.@id"];
              const contactInterval = setInterval(async () => {
                let retry = false;
                let userContacts = [];
                if (
                  userProfile &&
                  (await userProfile["contacts.ldp:contains"]) &&
                  (await userProfile["contacts.ldp:contains"]) !== "undefined"
                ) {
                  for (let contact of await userProfile[
                    "contacts.ldp:contains"
                  ]) {
                    if (contact) {
                      userContacts.push(await contact["contact.@id"]);
                    } else {
                      retry = true;
                    }
                  }
                  if (!retry) {
                    clearInterval(contactInterval);
                    if (!userContacts.includes(this.resource["@id"])) {
                      core.store.post(
                        {
                          contact: this.resource["@id"],
                          "@context": this.context,
                        },
                        contactsURL
                      );
                    }
                  }
                }
              }, 100);
            }
            await converse_sib.connected_deferred;

            this.importCSS(style);

            window.dispatchEvent(
              new CustomEvent("sib-change-chat", {
                detail: {
                  jid: this.jid,
                  is_groupchat: await this.resource.jabberRoom,
                  root: this.element.shadowRoot,
                },
              })
            );

            window.dispatchEvent(
              new CustomEvent("read", {
                detail: {
                  resource: {
                    "@id": this.resource["@id"],
                  },
                },
              })
            );
          }
        } // Else, not logged in, on chat change
      },

      removeConverseDatabase() {
        for (let key in sessionStorage) {
          if (sessionStorage.hasOwnProperty(key) && /converse/.test(key)) {
            sessionStorage.removeItem(key);
          }
        }
        for (let key in localStorage) {
          if (localStorage.hasOwnProperty(key) && /converse/.test(key)) {
            localStorage.removeItem(key);
          }
        }
        // indexedDB.deleteDatabase("converse-persistent");
      },

      initializeConverse() {
        window.converse_sib = {};

        // Deferred resolved at the end of this function
        converse_sib.loaded_deferred = new utils.Deferred();

        // Deferred resolved after converse.initialize
        converse_sib.connected_deferred = new utils.Deferred();

        this.removeConverseDatabase();

        // Initialize deferred resolution plugin
        setTimeout(async () => {
          // Initialize deferred resolution plugin
          converse.plugins.add("sib-connected", {
            initialize() {
              const { api } = this._converse;
              api.listen.on(
                "connectionInitialized",
                converse_sib.connected_deferred.resolve
              );
            },
          });

          const user = await document.querySelector("sib-auth").getUser();
          const userProfile = await core.store.getData(
            user["@id"],
            this.context
          );
          const jabberID = await userProfile["chatProfile.jabberID"];

          if (this.resource) {
            if (await this.resource.jabberRoom) {
              this.jid = await this.resource["jabberID"];
            } else {
              this.jid = await this.resource["chatProfile.jabberID"];
            }
          } else {
            this.jid = jabberID;
          }

          const idToken = await document
            .querySelector("sib-auth")
            .getUserIdToken();

          let lang = this.element.dataset.i18n || "fr";
          if (
            localStorage.getItem("language") ||
            (window.navigator.language !== null &&
              window.navigator.language !== undefined)
          ) {
            lang =
              localStorage.getItem("language") ||
              window.navigator.language.slice(0, 2);
          }

          let converseSettings = {
            assets_path: `${utils.path()}/src/conversejs/`,
            authentication: this.element.dataset.authentication || "login",
            password: idToken,
            allow_bookmarks: false,
            allow_chat_pending_contacts: true,
            allow_message_retraction: "own",
            allow_non_roster_messaging: true,
            allow_dragresize: false,
            allow_logout: false,
            archived_messages_page_size: "30",
            auto_list_rooms: true,
            auto_login: this.element.dataset.autoLogin === "true",
            auto_join_on_invite: true,
            auto_reconnect: true,
            auto_register_muc_nickname: false,
            websocket_url:
              this.element.dataset.websocketUrl ||
              "wss://jabber.happy-dev.fr/xmpp-websocket",
            enable_smacks: true,
            clear_messages_on_reconnection: false,
            discover_connection_methods: false,
            jid: jabberID.toLowerCase(),
            i18n: lang,
            loglevel: "info",
            message_archiving: "always",
            message_archiving_timeout: 60000,
            muc_disable_slash_commands: true,
            muc_hats: ["hats", "vcard_roles", "admin"],
            role_affiliation_hat_conversions: { admin: "Administrateur" },
            muc_nickname_from_jid: false,
            muc_fetch_members: true,
            muc_show_info_messages: [],
            muc_subscribe_to_rai: true,
            play_sounds: false,
            root: this.element.shadowRoot,
            show_client_info: false,
            show_desktop_notifications: false,
            persistent_store: "IndexedDB",
            sounds_path: `${utils.path()}/src/conversejs`,
            show_send_button: true,
            view_mode: "fullscreen",
            visible_toolbar_buttons: {
              call: false,
              spoiler: false,
              emoji: true,
              fileupload: true, // Not working in current Converse
              toggle_occupants: false,
            },
            whitelisted_plugins: [
              "converse-rai",
              "sib-action-buttons-inline",
              "sib-chat-navigation",
              "sib-connected",
              "sib-custom-hats",
              "sib-disconnected",
              "sib-emoji-picker",
              "sib-history-improved",
              "sib-mention-autocomplete",
              "sib-mention-mobile",
              "sib-reactions",
              "sib-remove-notifications",
              "sib-reply-to-message",
              "sib-retract-file-upload",
              "sib-scroll-down-on-focus",
              "sib-send-button-mobile",
              "sib-subscribe-to-rai",
            ],
          };

          // Debug mode (expose private converse api)
          if (this.element.dataset.debug === "true") {
            converse.plugins.add("sib-debug", {
              initialize() {
                window._converse = this._converse;
              },
            });
            converseSettings["whitelisted_plugins"].push("sib-debug");
            converseSettings["loglevel"] = "debug";
          }

          await converse.initialize(converseSettings);

          converse_sib.loaded_deferred.resolve();
        }, 0);
      },
    });
  });
});
